"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockoonServerless = void 0;
const commons_1 = require("@mockoon/commons");
const commons_server_1 = require("@mockoon/commons-server");
const serverless_http_1 = __importDefault(require("serverless-http"));
class MockoonServerless {
    constructor(environment, options) {
        this.environment = environment;
        this.options = {
            logTransaction: false,
            disabledRoutes: [],
            fakerOptions: {},
            envVarsPrefix: commons_1.defaultEnvironmentVariablesPrefix,
            enableAdminApi: true,
            disableTls: false,
            maxTransactionLogs: commons_1.defaultMaxTransactionLogs
        };
        if (!environment) {
            throw new Error('No environment data provided');
        }
        this.options = Object.assign(Object.assign({}, this.options), options);
    }
    /**
     * Returns an Express request listener that can be used in serverless environments
     * and with the serverless-http library.
     *
     * @returns
     */
    requestListener() {
        const logger = (0, commons_server_1.createLoggerInstance)();
        const server = new commons_server_1.MockoonServer(this.environment, {
            disabledRoutes: this.options.disabledRoutes,
            fakerOptions: this.options.fakerOptions,
            envVarsPrefix: this.options.envVarsPrefix,
            enableAdminApi: this.options.enableAdminApi,
            disableTls: this.options.disableTls,
            maxTransactionLogs: this.options.maxTransactionLogs
        });
        (0, commons_server_1.listenServerEvents)(server, this.environment, logger, this.options.logTransaction);
        return server.createRequestListener();
    }
    /**
     * Returns a serverless-http wrapped request listener for AWS Lambda.
     *
     * @returns
     */
    awsHandler() {
        return (0, serverless_http_1.default)(this.requestListener());
    }
    /**
     * Returns a request listener for Google Cloud Functions.
     *
     * @returns
     */
    firebaseApp() {
        return this.requestListener();
    }
    /**
     * Returns a serverless-http wrapped request listener for Netlify.
     *
     * @returns
     */
    netlifyHandler() {
        return (0, serverless_http_1.default)(this.requestListener());
    }
}
exports.MockoonServerless = MockoonServerless;
//# sourceMappingURL=serverless.js.map